"""Replace resets after measure with a conditional XGate."""

from qiskit.transpiler.basepasses import TransformationPass
from qiskit.dagcircuit import DAGCircuit
from qiskit.circuit.reset import Reset


class CollectResets(TransformationPass):
    def run(self, dag: DAGCircuit) -> DAGCircuit:
        # This method will be called on the DAG of the quantum circuit
        # to perform the transformation.
        
        # Iterate over all qubits in the circuit
        for qubit in dag.qubits:
            # Keep track of previous reset nodes for each qubit
            prev_reset_node = None
            
            # Get the nodes in topological order for processing
            for node in dag.topological_op_nodes():
                # Check if the node is a reset operation on the current qubit
                if isinstance(node.op, Reset) and node.qargs[0] == qubit:
                    if prev_reset_node:
                        # Merge this reset with the previous by removing the current node
                        dag.remove_op_node(node)
                    else:
                        # Update the previous reset node
                        prev_reset_node = node
                else:
                    # Reset prev_reset_node if the operation is not a reset
                    # or it's applied to a different qubit
                    prev_reset_node = None
        
        return dag


from qiskit.transpiler.preset_passmanagers.plugin import PassManagerStagePlugin
from qiskit.transpiler import PassManager
from dynacir.dynacir_passes import CollectResets

class DynacirPlugin(PassManagerStagePlugin):
    def pass_manager(self, pass_manager_config, optimization_level):
        pm = PassManager(
            [
                CollectResets()
            ]
        )
        return pm


import numpy as np
from qiskit_aer.noise import NoiseModel, errors
from qiskit.quantum_info.operators import Operator

def get_secret_noise_model():
    # Create the noise model
    noise_model = NoiseModel()

    # Add readout error
    readout_error = errors.readout_error.ReadoutError([[0.99, 0.01], [0.05, 0.95]])
    noise_model.add_all_qubit_readout_error(readout_error)

    # Add coherent error
    unitary_operator = Operator([[1, 0],[0, np.exp(1j*0.001)]])
    error_gate = errors.coherent_unitary_error(unitary_operator)
    noise_model.add_all_qubit_quantum_error(error_gate, ['rz', 'sx'], warnings=False)

    # Add depolarizing error
    depolarizing_error = errors.depolarizing_error(0.005, 2)
    noise_model.add_all_qubit_quantum_error(depolarizing_error, "cx")


from setuptools import setup, find_packages

found_packages = find_packages()
print("Found packages:", found_packages)

setup(
    name='dynacir',
    version='0.1.2',
    packages=find_packages(),
)
